#
#   PROBE - The central game object
#

import random
from albow.resource import get_sound, get_text
from system import System
from settings import frame_time, options
from vector_algebra import add, rectangular
from rocket_science import fuel_for_liftoff
from data import DatumDict
from design import ProbeDesign
import instruments

class Game(object):
	#  current_system       System
	#  mission_clock        float, s
	#  current_time_scale   float
	#  collected_data       DatumDict{(system_name, planet_name, category): Datum}
	#  designs              [ProbeDesign]
	#  current_design       ProbeDesign
	#  budget               number, dollars
	#  revenue              number, dollars
	#  funding_result_text  string
	
	launch_sound = get_sound("countdown-and-launch.ogg")
	time_scales = [1, 10, 60, 600, 3600, 36000, 86400, 864000]
	fuel_price = 0.50 # $/kg
	
	current_system = None
	mission_clock = -5.0
	mission_clock_running = False
	current_time_scale = 1.0
	current_design = None
	revenue = 0
	funding_result_text = "This is the funding result." ###
	
	def __init__(self):
		random.seed(options.random_seed)
		self.collected_data = DatumDict()
		self.designs = []
		self.budget = 1000000
		self.new_design()

	def new_system(self):
		self.current_system = System()
		if options.test_design:
			import design
			self.current_design = design.create_test_design()
		if options.test_vehicle:
			print "Game.new_system: creating test vehicle" ###
			home = self.current_system.home
			self.current_system.set_vehicle(create_test_vehicle(home))
		if options.test_data:
			print "Game.new_system: creating test data" ###
			import data
			self.store_datum(home, data.TemperatureDatum(42, 0.5))
			self.store_datum(home, data.AtmosphericPressureDatum(42, 0.4))
	
	def new_design(self):
		design = ProbeDesign()
		self.designs.append(design)
		self.current_design = design

	def launch(self):
		if not self.mission_clock_running:
			self.mission_clock_running = True
			if options.test_skip_countdown:
				self.mission_clock = 0.0
			else:
				self.launch_sound.play()
	
	def begin_frame(self):
		dt = frame_time * 1e-3 * self.current_time_scale
		old_clock = self.mission_clock
		new_clock = old_clock
		if self.mission_clock_running:
			new_clock += dt
			self.mission_clock = new_clock
		system = self.current_system
		if system:
			if old_clock <= 0.0 and new_clock > 0.0:
				system.end_countdown()
			system.timestep(self, dt)
	
	def num_timescales(self):
		return len(self.time_scales)
	
	def current_timescale_number(self):
		return self.time_scales.index(self.current_time_scale)
	
	def set_timescale_number(self, n):
		n = max(0, min(self.num_timescales() - 1, n))
		self.current_time_scale = self.time_scales[n]
	
	def decrease_timescale(self):
		self.set_timescale_number(self.current_timescale_number() - 1)

	def increase_timescale(self):
		self.set_timescale_number(self.current_timescale_number() + 1)
	
	def get_datum(self, planet, category):
		#  Get datum about planet of current system
		key = (self.current_system.name, planet.name, category)
		return self.collected_data.get(key)
	
	def store_datum(self, planet, datum):
		#  Stores datum under the name of the current system.
		system = self.current_system
		home = system.home
		key = (system.name, planet.name, datum.category)
		self.collected_data.store(key, datum, pre_rewarded = planet is home)
	
	def buy_fuel(self, fuel):
		cost = int(fuel * self.fuel_price)
		if cost > self.budget:
			raise FuelCostError(cost)
		self.budget -= cost
		
	def request_funding(self, requested_amount):
		print "Game.request_funding:", requested_amount ###
		self.revenue += self.gather_revenue()
		if self.revenue >= requested_amount:
			allocated_amount = requested_amount
			text = get_text("fully_granted_fund_app.txt")
		elif self.revenue > 0:
			allocated_amount = self.revenue
			text = get_text("partially_granted_fund_app.txt")
		else:
			allocated_amount = 0
			text = get_text("rejected_fund_app.txt")
		self.budget += allocated_amount
		self.revenue -= allocated_amount
		self.funding_result_text = text % dict(
			REQUESTED = requested_amount,
			ALLOCATED = allocated_amount,
			BUDGET = self.budget)

	def gather_revenue(self):
		return self.collected_data.gather_revenue()


class FuelCostError(StandardError):

	def __init__(self, cost):
		self.cost = cost
		StandardError.__init__(self, "Can't afford fuel cost of $%s" % cost)


def create_test_vehicle(home):
	from vehicle import Vehicle, VehicleStage
	vehicle = Vehicle("Pork Pie 1")
	vehicle.add_instrument(instruments.TemperatureSensor(1e10, 0.20))
	vehicle.add_instrument(instruments.AtmosphericPressureSensor(1e10, 0.10))
	vehicle.add_instrument(instruments.Spectrometer(1e10, 0.10, 0.05))
	#                               m0  Isp      b    f    tb
	vehicle.add_stage(VehicleStage(1000, 300,    1, 1e3))
	vehicle.add_stage(VehicleStage(1000, 350, 10e3,   0, 1e30))
	vehicle.name_stages()
	if options.test_prepare:
		booster = vehicle.stages[-1]
		b = booster.burn_rate
		m = vehicle.mass
		g = home.surface_gravity()
		ve = home.escape_velocity()
		v = 20e3 # ve
		a = -84
		fuel = fuel_for_liftoff(booster.Isp, b, m, g, v, 0.01)
		booster.fuel_capacity = 10 * fuel
		vehicle.prepare_for_launch(home, heading = a, fuel = fuel)
	return vehicle
